﻿//////////////////////////////////////////////////////////////////////////////
//
//  Motion Patterns Builder  ( for PCL6115-EV )
//  Version = 1.20 (2019.12.27)
//
//  Generate C# source file.
//  2019/12/27 15:30:55
//
//  NIPPON PULSE MOTOR Co., Ltd.
//
//////////////////////////////////////////////////////////////////////////////

using FTD2XX_NET;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;


namespace PCL6115_EV2S
{
    public class samplePCL6115EV2S
    {
        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Make the GetInputState-function available
        /// </summary>
        [DllImport("user32")]
        public static extern long GetInputState();
        [DllImport("winmm.dll", EntryPoint = "timeGetTime")]
        public static extern uint GetTime();

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// class / table
        /// </summary>
        private accessPCL6115 hPCL6115;

        //Flag definition
        private bool FlagZero = false;
        private bool FlagOver = false;
        // レジスタ定義
        private ushort REG_00;
        private ushort REG_01;

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Clear Register.
        /// </summary>
        private void ClearReg()
        {
            REG_00 = 0;
            REG_01 = 0;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// If there is an Windows-event, process it.
        /// </summary>
        private void CheckWindowsEvent()
        {
            if ((GetInputState() & 0x00000000FFFFFFFF) != 0)
            {
                System.Windows.Forms.Application.DoEvents();
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Register manipulation function
        /// </summary>
        public void RegCalculation(string type, ref ushort Data1, ushort Data2)
        {
            int workData;

            CheckWindowsEvent();
            //
            if (hPCL6115.PCL6115EV_stop == false)//中断フラグがオフのときのみ実行
            {
                if(type == "IN") // Assign data
                {
                    workData = (int)Data2;
                }
                else if(type == "ADD") // Addition
                {
                    workData = (int)(Data1) + (int)(Data2);
                }
                else // if(type == "SUB") // Subtraction
                {
                    workData = (int)(Data1) - (int)(Data2);
                }
                //
                if((workData & 0xFFFF0000) == 0)
                {
                    FlagOver = false;
                }
                else
                {
                    FlagOver = true;
                }
                //
                if((workData & 0x0000FFFF) == 0)
                {
                    FlagZero = true;
                }
                else
                {
                    FlagZero = false;
                }
                //
                Data1 = (ushort)(workData & 0x0000FFFF);
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Status state wait function
        /// type=0：Wait for Main-status
        ///      1：Wait for Sub-status
        /// </summary>
        public void WaitMsts(uint mask, uint WaitValue, int type)
        {
            uint MainStatus = 0;
            uint SubStatus = 0;
            uint WorkData;

            WorkData = ~WaitValue;
            //
            while((WorkData & mask) != WaitValue)
            {
                CheckWindowsEvent();
                //
                if (hPCL6115.PCL6115EV_stop == false)//中断フラグがオフのときのみ実行
                {
                    //
                    hPCL6115.Read_STATUS(ref MainStatus, ref SubStatus);
                    //
                    if(type ==0)
                    {
                        WorkData = MainStatus;
                    }
                    else
                    {
                        WorkData = SubStatus;
                    }
                }
                else
                {
                    hPCL6115.Write_COM((byte)0x49);//中断時はSTOPコマンドを書込む
                    hPCL6115.SendUsb();
                    break;
                }
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Register state wait function
        /// </summary>
        public void WaitReg(byte RegReadCom, uint mask, uint WaitValue)
        {
            uint ReadData;

            ReadData = ~WaitValue;
            //
            while((ReadData & mask) != WaitValue)
            {
                if (hPCL6115.PCL6115EV_stop == false)//中断フラグがオフのときのみ実行
                {
                    CheckWindowsEvent();
                    //
                    hPCL6115.Read_REG((byte)RegReadCom, ref ReadData);
                }
                else
                {
                    hPCL6115.Write_COM((byte)0x49);//中断時はSTOPコマンドを書込む
                    hPCL6115.SendUsb();
                    break;
                }
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Time wait function
        /// </summary>
        public void WaitTime(int TimeValue)
        {
            uint Tick;

            Tick = GetTime() + (uint)TimeValue;
            while (GetTime() < Tick)
            {
                CheckWindowsEvent();
                if (hPCL6115.PCL6115EV_stop == true) break;//中断
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Main processing
        ///   Return value
        ///      0 : All processing was completed.
        ///     -1 : Processing was interrupted.
        /// </summary>
        public int main(accessPCL6115 hPCL)
        {
            hPCL6115 = hPCL;
            ClearReg();

            //
            if(hPCL6115.PCL6115EV_on == true)
            {
                // ------------------------------------------------------
                // 0 [ 実装ドライバIC(TB6608FNG)対応パルスモード ]
                //
                hPCL6115.Write_REG(hPCL6115.WRENV1, 0x00000002);
                hPCL6115.SendUsb();

                // ------------------------------------------------------
                // 1 [ 実装ドライバICの励磁モード2W1-2相 ]
                //
                hPCL6115.Write_COM((byte)0x13);
                hPCL6115.Write_COM((byte)0x14);
                hPCL6115.SendUsb();

                // ------------------------------------------------------
                // 2 [ 実装ドライバICが動作可能 ]
                //
                hPCL6115.Write_COM((byte)0x15);
                hPCL6115.Write_COM((byte)0x16);
                hPCL6115.Write_COM((byte)0x17);
                hPCL6115.SendUsb();

                // ------------------------------------------------------
                // 3 [ 0番目のレジスタの初期設定 ]
                //
            LOOP1:
                RegCalculation("IN", ref REG_00, 3);

                // ------------------------------------------------------
                // 4 [ ﾏｲﾅｽ方向に半回転(1152パルス)[原点復帰] ]
                //
                hPCL6115.Write_REG(hPCL6115.WPRMV, 0xFFFFFB80);
                hPCL6115.Write_REG(hPCL6115.WPRFL, 0x000000C8);
                hPCL6115.Write_REG(hPCL6115.WPRFH, 0x000007D0);
                hPCL6115.Write_REG(hPCL6115.WPRUR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRDR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMG, 0x000004AF);
                hPCL6115.Write_REG(hPCL6115.WPRDP, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMD, 0x00000041);
                hPCL6115.Write_COM((byte)0x50);
                hPCL6115.SendUsb();
                // Wait until operation is completed. [ エラーに対する処理は省略しています。 ]
                WaitMsts(0x0000000B, 0x00000008, 0); // MainStatus check

                // ------------------------------------------------------
                // 5 [ CW方向へ半回転(1152パルス)／一定速度動作 ]
                //
                hPCL6115.Write_REG(hPCL6115.WPRMV, 0x00000480);
                hPCL6115.Write_REG(hPCL6115.WPRFL, 0x00000190);
                hPCL6115.Write_REG(hPCL6115.WPRFH, 0x00000190);
                hPCL6115.Write_REG(hPCL6115.WPRUR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRDR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMG, 0x000004AF);
                hPCL6115.Write_REG(hPCL6115.WPRDP, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMD, 0x00000041);
                hPCL6115.Write_COM((byte)0x51);
                hPCL6115.SendUsb();
                // Wait until operation is completed. [ エラーに対する処理は省略しています。 ]
                WaitMsts(0x0000000B, 0x00000008, 0); // MainStatus check

                // ------------------------------------------------------
                // 6 [ 1番目のレジスタの初期設定 ]
                //
            LOOP2:
                RegCalculation("IN", ref REG_01, 2);

                // ------------------------------------------------------
                // 7 [ CW方向へ1回転(2304パルス)／直線加減速 ]
                //
            LOOP3:
                hPCL6115.Write_REG(hPCL6115.WPRMV, 0x00000900);
                hPCL6115.Write_REG(hPCL6115.WPRFL, 0x00000190);
                hPCL6115.Write_REG(hPCL6115.WPRFH, 0x000005DC);
                hPCL6115.Write_REG(hPCL6115.WPRUR, 0x000006FA);
                hPCL6115.Write_REG(hPCL6115.WPRDR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMG, 0x000004AF);
                hPCL6115.Write_REG(hPCL6115.WPRDP, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMD, 0x00000041);
                hPCL6115.Write_COM((byte)0x53);
                hPCL6115.SendUsb();
                // Wait until operation is completed. [ エラーに対する処理は省略しています。 ]
                WaitMsts(0x0000000B, 0x00000008, 0); // MainStatus check

                // ------------------------------------------------------
                // 8 [ 500msのウェイト時間 ]
                //
                WaitTime(500); // 精度の期待はできません。

                // ------------------------------------------------------
                // 9 [ 1番目のレジスタ内容を１減算する ]
                //
                RegCalculation("SUB", ref REG_01, 1);

                // ------------------------------------------------------
                // 10 [ REG1の内容が0でない時、LOOP3へ ]
                //
                if(hPCL6115.PCL6115EV_stop == true)
                {
                   goto Forced_termination;
                }
                else if(FlagZero == false)
                {
                    goto LOOP3;
                }

                // ------------------------------------------------------
                // 11 [ CCW方向へ2回転(4608パルス)／Ｓ字加減速 ]
                //
                hPCL6115.Write_REG(hPCL6115.WPRMV, 0xFFFFEE00);
                hPCL6115.Write_REG(hPCL6115.WPRFL, 0x00000190);
                hPCL6115.Write_REG(hPCL6115.WPRFH, 0x000005DC);
                hPCL6115.Write_REG(hPCL6115.WPRUR, 0x0000037C);
                hPCL6115.Write_REG(hPCL6115.WPRDR, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMG, 0x000004AF);
                hPCL6115.Write_REG(hPCL6115.WPRDP, 0x00000000);
                hPCL6115.Write_REG(hPCL6115.WPRMD, 0x00000441);
                hPCL6115.Write_COM((byte)0x53);
                hPCL6115.SendUsb();
                // Wait until operation is completed. [ エラーに対する処理は省略しています。 ]
                WaitMsts(0x0000000B, 0x00000008, 0); // MainStatus check

                // ------------------------------------------------------
                // 12 [ 1000msのウェイト時間 ]
                //
                WaitTime(1000); // 精度の期待はできません。

                // ------------------------------------------------------
                // 13 [ 0番目のレジスタ内容を１減算する ]
                //
                RegCalculation("SUB", ref REG_00, 1);

                // ------------------------------------------------------
                // 14 [ REG0の内容が0でない時、LOOP2へ ]
                //
                if(hPCL6115.PCL6115EV_stop == true)
                {
                   goto Forced_termination;
                }
                else if(FlagZero == false)
                {
                    goto LOOP2;
                }

                // ------------------------------------------------------
                // 15 [ 無条件でLOOP1へ ]
                //
                goto LOOP1;

            } // if(hPCL6115.PCL6115EV_on == true)
        Forced_termination:
            if (hPCL6115.PCL6115EV_stop == true)//中断
            {
                return (-1);
            }
            hPCL6115.PCL6115EV_stop = false;
            return (0);//全て終了
        }
    }
}
